//
// Copyright 2020 Electronic Arts Inc.
//
// TiberianDawn.DLL and RedAlert.dll and corresponding source code is free 
// software: you can redistribute it and/or modify it under the terms of 
// the GNU General Public License as published by the Free Software Foundation, 
// either version 3 of the License, or (at your option) any later version.

// TiberianDawn.DLL and RedAlert.dll and corresponding source code is distributed 
// in the hope that it will be useful, but with permitted additional restrictions 
// under Section 7 of the GPL. See the GNU General Public License in LICENSE.TXT 
// distributed with this program. You should have received a copy of the 
// GNU General Public License along with permitted additional restrictions 
// with this program. If not, see https://github.com/electronicarts/CnC_Remastered_Collection

/* $Header: /CounterStrike/CRCPIPE.CPP 1     3/03/97 10:24a Joe_bostic $ */
/***********************************************************************************************
 ***              C O N F I D E N T I A L  ---  W E S T W O O D  S T U D I O S               ***
 ***********************************************************************************************
 *                                                                                             *
 *                 Project Name : Command & Conquer                                            *
 *                                                                                             *
 *                    File Name : CRCPIPE.CPP                                                  *
 *                                                                                             *
 *                   Programmer : Joe L. Bostic                                                *
 *                                                                                             *
 *                   Start Date : 06/30/96                                                     *
 *                                                                                             *
 *                  Last Update : July 3, 1996 [JLB]                                           *
 *                                                                                             *
 *---------------------------------------------------------------------------------------------*
 * Functions:                                                                                  *
 *   CRCPipe::Result -- Fetches the current CRC of the data.                                   *
 *   CRCPipe::Put -- Retrieves the data bytes specified and calculates CRC on it.              *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */


#include "crcpipe.h"


/***********************************************************************************************
 * CRCPipe::Put -- Retrieves the data bytes specified and calculates CRC on it.                *
 *                                                                                             *
 *    This routine will fetch the number of bytes requested from the straw. The data is        *
 *    not modified by this straw segment, but it is examined by the CRC engine in order to     *
 *    keep an accurate CRC of the data that passes through this routine.                       *
 *                                                                                             *
 * INPUT:   source   -- Pointer to the buffer that will hold the data requested.               *
 *                                                                                             *
 *          length   -- The number of bytes requested.                                         *
 *                                                                                             *
 * OUTPUT:  Returns with the actual number of bytes stored into the buffer. If this number is  *
 *          less than the number requested, then this indicates that the data stream has been  *
 *          exhausted.                                                                         *
 *                                                                                             *
 * WARNINGS:   none                                                                            *
 *                                                                                             *
 * HISTORY:                                                                                    *
 *   07/03/1996 JLB : Created.                                                                 *
 *=============================================================================================*/
int CRCPipe::Put(void const * source, int slen)
{
	CRC(source, slen);
	return(Pipe::Put(source, slen));
}


/***********************************************************************************************
 * CRCPipe::Result -- Fetches the current CRC of the data.                                     *
 *                                                                                             *
 *    This routine will return the CRC of the data that has passed through the pipe up to      *
 *    this time.                                                                               *
 *                                                                                             *
 * INPUT:   none                                                                               *
 *                                                                                             *
 * OUTPUT:  Returns with the CRC value.                                                        *
 *                                                                                             *
 * WARNINGS:   none                                                                            *
 *                                                                                             *
 * HISTORY:                                                                                    *
 *   07/03/1996 JLB : Created.                                                                 *
 *=============================================================================================*/
long CRCPipe::Result(void) const
{
	return(CRC());
}

